SuperStrict

Import "TTile.bmx"

Type T2DMap
	Field width:int, height:int
	field tiles:TTile[,]

	Function Create:T2DMap(w:int, h:int)
		Local o:T2DMap = new T2DMap
			o.width = w
			o.height = h
			o.tiles = new TTile[w,h]
			For Local y:Int = 0 Until h
				Local d:int = y * 8.0
				Local color:TColor = TColor.Create(0, d, 255)
				For Local x:Int = 0 Until w
					o.tiles[x,y] = TTile.Create("X", color)
				Next
			Next
		return o
	EndFunction

	Method Save()
		Local date:String = CurrentDate() + "-" + CurrentTime()
		date = Replace(date, ":", "-")
		date = Replace(date, " ", "-")
		Local fileout:TStream = WriteStream("map-" + date + ".txt")
		fileout.WriteLine("")
		For Local y:Int = 0 Until height
			Local line:String = ""
			For Local x:Int = 0 Until width
				line :+ tiles[x,y].s
			Next
			fileout.WriteLine(line)
		Next
		fileout.WriteLine("")
		fileout.Close()
	EndMethod

	Method Draw(xoffset:int=0, yoffset:int=0, size:int=10)
		For Local y:Int = 0 Until height
			For Local x:Int = 0 Until width
				tiles[x,y].Draw(xoffset+x*size, yoffset+y*size)
			Next
		Next
	EndMethod

	Method Earth()
		Local EarthTiles:TTile[]
		EarthTiles:+[TTile.Create("~~", TColor.Create(64, 64, 255))] ' Water
		EarthTiles:+[TTile.Create("#", TColor.Create(64,255,64))] ' green land
		EarthTiles:+[TTile.Create("+", TColor.Create(64,200,64))] ' green land
		EarthTiles:+[TTile.Create("=", TColor.Create(200,200,200))] ' Ice/snow

		RestoreData DataBlockEarth
		self.LoadDataBlock(EarthTiles)
	EndMethod


	Method Test()
		Local TestTiles:TTile[]
		TestTiles:+[TTile.Create("X", TColor.Create(128, 128, 128))] ' Gray
		TestTiles:+[TTile.Create("R", TColor.Create(128, 64, 64))] ' red
		TestTiles:+[TTile.Create("G", TColor.Create(64, 128, 64))] ' green
		TestTiles:+[TTile.Create("B", TColor.Create(64, 64, 128))] ' blue
		TestTiles:+[TTile.Create("?", TColor.Create(128, 0, 255))] ' lila

		RestoreData DataBlockTest
		self.LoadDataBlock(TestTiles)
	EndMethod

	Method LoadDataBlock(_tiles:TTile[])
		For Local y:Int = 0 Until height
			For Local x:Int = 0 Until width
				Local i:int = 0
				ReadData i
				tiles[x,y] = _tiles[i]
			Next
		Next
	EndMethod

	Method randTile:TTile(list:TTile[])
		return list[Rand(0, list.length-1)]
	EndMethod

	Method randPosition:Int[] (list:int[][])
		return list[Rand(0, list.length-1)]
	EndMethod

	Method randNeigbour:Int[] (p:int[])
		Local rx:int = 0
		Local ry:int = 0
		Repeat
			rx = rand(-1,1)
			ry = rand(-1,1)
		Until (rx+ry <> 0)
		Local x:int = p[0] + rx
		Local y:int = p[1] + ry
		x = (x+width) mod width
		y = (y+height) mod height
		return [x,y]
	EndMethod

	Method getNeigbours:Int[][] (p:int[])
		Local result:Int[][]
		For Local y1:Int = -1 To 1
			For Local x1:Int = -1 To 1
				if x1 = 0 and y1 = 0 then continue
				Local x:Int = p[0] + x1
				Local y:Int = p[1] + y1
				x = (x+width) mod width
				y = (y+height) mod height
				result:+[[x, y]]
			Next
		Next

		return result
	EndMethod

	Method LineH(y:int, t:TTile=null)
		if not t then t = TTile.Create("E", TColor.Create(200,50,50))
		For Local x:Int = 0 Until width
			tiles[x,y] = t
		Next
	EndMethod

	Method Fill(t:TTile)
		For Local x:Int = 0 Until width
			For Local y:Int = 0 Until height
				tiles[x,y] = t
			Next
		Next
	EndMethod

	Method Generate(typ:int=-1)
		self.GenerateEarthLike()
		'self.GenerateGasGiant() '' really boring

		Rem
		if typ = -1 then typ = rand(0,1)
		Select typ
			Case 1
			Default
		EndSelect
		EndRem
	EndMethod

	Method GenerateEarthLike()
		Local water:TTile = TTile.Create("~~", TColor.Create(64, 64, 255))
		Local land:TTile = TTile.Create("#", TColor.Create(64,255,64))
		Local land2:TTile = TTile.Create("+", TColor.Create(64,255,64))
		Local land3:TTile = TTile.Create(",", TColor.Create(64,255,64))
		self.Fill( water )

		'Begin with some random positions as land
		Local landlist:Int[][]
		For Local i:Int = 0 Until rand(3,9)
			Local x:int = rand(0,self.width-1)
			Local y:int = rand(0,self.height-1)
			landlist:+[[x, y]]
			tiles[x, y] = land
		Next

		'Grow islands into continents by selecting neigbouring tiles
		Local percentageLand:float = rnd(0.2, 0.6)
		Local tilesMax:int = width*height*percentageLand
		While (landlist.length < tilesMax)
			local p:int[] = self.randPosition(landlist)
			p = self.randNeigbour(p)
			if tiles[p[0], p[1]] <> land then
				landlist:+[[p[0], p[1]]]
				tiles[p[0], p[1]] = land
			endif
		Wend

		'Sligtly modify coast
		For Local y:Int = 0 Until height
			For Local x:Int = 0 Until width
				if tiles[x, y] = land then
					Local watertiles:int = 0
					For Local n:Int[] = eachin self.getNeigbours([x, y])
						if tiles[n[0], n[1]] = water then watertiles:+1
					Next
					if watertiles => 3 then tiles[x, y] = land2
					If watertiles => 6 then tiles[x, y] = land3
				endif
			Next
		Next

		self.AddPoles(Rand(Rand(0,16)))
	EndMethod

	Method AddPoles(size:int=1)
		local snowcolor:TColor = TColor.Create(200,200,200)
		Local snow:TTile = TTile.Create("=", snowcolor)

		For Local x:Int = 0 Until width
			For Local y:Int = 0 Until size
				if tiles[x,y].s = "~~" then
					tiles[x,y] = snow
				Else
					tiles[x,y] = TTile.Create(tiles[x,y].s, snowcolor)
				endif
			Next
			For Local y:Int = height-size Until height
				if tiles[x,y].s = "~~" then
					tiles[x,y] = snow
				Else
					tiles[x,y] = TTile.Create(tiles[x,y].s, snowcolor)
				endif
			Next
		Next
	EndMethod

	Method GenerateGasGiant()
		Local _tiles:TTile[]
		_tiles:+[TTile.Create("8", TColor.Create(128, 64, 64))]
		_tiles:+[TTile.Create("#", TColor.Create(64, 64, 128))]
		_tiles:+[TTile.Create("V", TColor.Create(128, 64, 128))]
		_tiles:+[TTile.Create("K", TColor.Create(250, 128, 128))]
		_tiles:+[TTile.Create("M", TColor.Create(64, 150, 64))]

		self.Fill(self.randTile(_tiles))

		For Local i:Int = 0 Until 50
			self.LineH(Rand(0, height-1), self.randTile(_tiles))
		Next
	EndMethod

EndType




#DataBlockEarth
DefData 3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3
DefData 3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,2,2,2,2,2,2,2,2,0,0,2,2,1,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,2,2,1,2,2,0,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,2,2,2,2,0,2,2,2,2,1,2,2,2,2,1,1,2,0,0,2,1,1,1,1,1,0,0,0,0,0,0,0,2,2,2,0,0,0,2,2,0,1,2,1,1,1,1,1,1,1,1,1,2,2,1,1,2,2,2,2,2
DefData 2,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,0,0,1,1,2,0,0,2,0,0,0,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
DefData 0,0,0,2,2,2,0,2,1,1,1,1,1,1,1,2,0,0,2,1,2,0,0,0,0,0,0,0,0,0,0,2,0,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,0,0
DefData 0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,1,1,2,1,1,1,1,0,0,0,0,0,0,0,0,2,1,0,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,0,2,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,1,1,1,2,2,2,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0,2,1,2,2,2,2,1,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,2,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,0,0,2,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,2,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,1,1,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,2,2,0,0,2,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,1,1,1,0,1,1,2,0,0,1,1,2,0,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,1,2,0,0,0,2,2,0,0,2,1,2,0,2,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,1,2,2,0,0,0,0,0,0,0,2,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,2,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,2,0,0,0,0,0,0,0,0,2,2,0,2,2,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,1,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0,2,0,2,2,0,2,2,2,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,2,1,1,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,2,1,1,1,1,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,0,0,0,0,0,0,0,0,0,2,1,1,1,2,0,2,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,2,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,2,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,1,1,1,2,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,1,2,1,1,1,2,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,3,0,0,0,0,0,0,0,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,0
DefData 0,0,0,0,0,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,0
DefData 3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3

#DataBlockTest
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,4,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,4,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,3,3,3,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,4,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,4,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,3,0,0,0,0,0,3,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,4,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,3,0,0,0,0,0,0,0,3,0,0,0,0,0,3,0,0,0,4,4,4,4,4,4,4,4,4,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,3,0,0,0,0,0,0,0,0,0,3,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,4,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,3,0,0,0,0,0,0,0,0,0,4,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,3,0,0,0,0,0,0,0,0,4,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,4,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,4,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,4,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
DefData 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
