SuperStrict

import "TTile.bmx"
Import "T2DMap.bmx"

Type TPlanet

	Field points:TPoint[]

	Function Create:TPlanet(map:T2DMap)
		Local o:TPlanet = New TPlanet
		'o.points = TPlanet.FibonacciSphere(points)

		For Local y:Int = 0 Until map.height
			For Local x:Int = 0 Until map.width
				' Move every other point half a "ring" up
				' lines aren't straight anymore, but the sphere is more neatly filled
				Local halfY:float = (x mod 2) * (180.0 / float(map.height) * 0.5)
				local alpha:float = (y / float(map.height) * 180.0) + halfY
				Local betha:Float = x / Float(map.width) * 360.0
				Local px:Float = Sin(alpha) * sin(betha)
				Local py:Float = Sin(alpha) * cos(betha)
				Local pz:Float = Cos(alpha)
				o.points:+[TPoint.Create(px, py, pz, map.tiles[x, y])]
			Next
		Next

		o.Turn(80, 0, 0)
		Return o
	End Function

	Function FibonacciSphere:TPoint[] (samples:Int = 1)
		Local points:TPoint[]
		Local offset:Float = 2.0 / samples
		Local increment:Float = Pi * (3.0 - Sqr(5.0))
		Local random:Float = Rnd(0, 1) * samples

		For Local i:Int = 0 To samples
			Local y:Float = ((i * offset) - 1) + (offset / 2.0)
			Local r:Float = Sqr(1 - y ^ 2)
			Local phi:Float = ((i + random) Mod samples) * increment
			Local x:Float = Cos(phi) * r
			Local z:Float = Sin(phi) * r

			points:+[TPoint.Create(x, y, z, TTile.Create("X"))]

		Next

		Return points
	End Function

	Method Turn(x:Float, y:Float, z:Float)
		For Local p:TPoint = EachIn Self.points
			p.TurnX(x)
			p.TurnY(y)
			p.TurnZ(z)
		Next
	End Method

	Method Draw(x:Float, y:Float, scale:Float)
		For Local p:TPoint = EachIn Self.points
			p.Draw(x, y, scale)
		Next
	End Method

End Type


Type TPoint
	Field x:float, y:float, z:float
	Field tile:TTile

	Function Create:TPoint(x:Float, y:Float, z:Float, tile:TTile)
		Local o:TPoint = New TPoint
		o.x = x
		o.y = y
		o.z = z
		o.tile = tile
		Return o
	End Function

	Method TurnX(rotation:Float)
		Local ny:Float = y *cos(rotation) - z*sin(rotation)
		Local nz:Float = z * cos(rotation) + y*sin(rotation)
		y = ny
		z = nz
	End Method

	Method TurnY(rotation:Float)
		Local nx:Float = x *cos(rotation) - z*sin(rotation)
		Local nz:Float = z * cos(rotation) + x*sin(rotation)
		x = nx
		z = nz
	End Method

	Method TurnZ(rotation:Float)
		Local nx:Float = x *cos(rotation) - y*sin(rotation)
		Local ny:Float = y *cos(rotation) + x*sin(rotation)
		y = ny
		x = nx
	End Method

	Method Draw(x:Float, y:Float, scale:Float)
		If Self.z < 0 Then Return 'Do not draw "backside"
		SetScale(Self.z, Self.z)
		tile.Draw(Self.x * scale + x, Self.y * scale + y)
	End Method

End Type 